#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
use t::APISIX 'no_plan';

repeat_each(1);
no_long_string();
no_root_location();
no_shuffle();
log_level("info");

run_tests;

__DATA__

=== TEST 1: get route schema
--- request
GET /apisix/admin/schema/route
--- response_body eval
qr/"plugins":\{"type":"object"}/



=== TEST 2: get service schema and check if it contains `anyOf`
--- config
    location /t {
        content_by_lua_block {
            local core = require("apisix.core")
            local t = require("lib.test_admin").test
            local code, _, res_body = t('/apisix/admin/schema/service', ngx.HTTP_GET)
            local res_data = core.json.decode(res_body)
            if res_data["anyOf"] then
                ngx.say("found `anyOf`")
                return
            end

            ngx.say("passed")
        }
    }
--- request
GET /t
--- response_body
passed



=== TEST 3: get not exist schema
--- request
GET /apisix/admin/schema/noexits
--- error_code: 400



=== TEST 4: wrong method
--- request
PUT /apisix/admin/schema/service
--- error_code: 404



=== TEST 5: wrong method
--- request
POST /apisix/admin/schema/service
--- error_code: 404



=== TEST 6: ssl
--- config
location /t {
    content_by_lua_block {
        local ssl = require("apisix.schema_def").ssl
        local t = require("lib.test_admin").test
        local code, body = t('/apisix/admin/schema/ssl',
            ngx.HTTP_GET,
            nil,
            ssl
            )

        ngx.status = code
        ngx.say(body)
    }
}
--- request
GET /t
--- response_body
passed



=== TEST 7: get plugin's schema
--- request
GET /apisix/admin/schema/plugins/limit-count
--- response_body eval
qr/"required":\["count","time_window"\]/



=== TEST 8: get not exist plugin
--- request
GET /apisix/admin/schema/plugins/no-exist
--- error_code: 404



=== TEST 9: serverless-pre-function
--- config
location /t {
    content_by_lua_block {
        local t = require("lib.test_admin").test
        local code, body = t('/apisix/admin/schema/plugins/serverless-pre-function',
            ngx.HTTP_GET,
            nil,
            [[{
                "properties": {
                    "phase": {
                        "enum": ["rewrite", "access", "header_filter", "body_filter", "log", "before_proxy"],
                        "type": "string"
                    },
                    "functions": {
                        "minItems": 1,
                        "type": "array",
                        "items": {
                            "type": "string"
                        }
                    }
                },
                "required": ["functions"],
                "type": "object"
            }]]
            )

        ngx.status = code
        ngx.say(body)
    }
}
--- request
GET /t
--- response_body
passed



=== TEST 10: serverless-post-function
--- config
location /t {
    content_by_lua_block {
        local t = require("lib.test_admin").test
        local code, body = t('/apisix/admin/schema/plugins/serverless-post-function',
            ngx.HTTP_GET,
            nil,
            [[{
                "properties": {
                    "phase": {
                        "enum": ["rewrite", "access", "header_filter", "body_filter", "log", "before_proxy"],
                        "type": "string"
                    },
                    "functions": {
                        "minItems": 1,
                        "type": "array",
                        "items": {
                            "type": "string"
                        }
                    }
                },
                "required": ["functions"],
                "type": "object"
            }]]
            )

        ngx.status = code
        ngx.say(body)
    }
}
--- request
GET /t
--- response_body
passed



=== TEST 11: get plugin udp-logger schema
--- request
GET /apisix/admin/schema/plugins/udp-logger
--- response_body  eval
qr/"properties":/



=== TEST 12: get plugin grpc-transcode schema
--- request
GET /apisix/admin/schema/plugins/grpc-transcode
--- response_body eval
qr/("proto_id".*additionalProperties|additionalProperties.*"proto_id")/



=== TEST 13: get plugin prometheus schema
--- request
GET /apisix/admin/schema/plugins/prometheus
--- response_body eval
qr/"disable":\{"type":"boolean"\}/



=== TEST 14: get plugin node-status schema
--- extra_yaml_config
plugins:
    - node-status
--- request
GET /apisix/admin/schema/plugins/node-status
--- response_body eval
qr/"disable":\{"type":"boolean"\}/



=== TEST 15: get global_rule schema to check if it contains `create_time` and `update_time`
--- request
GET /apisix/admin/schema/global_rule
--- response_body eval
qr/("update_time":\{"type":"integer"\}.*"create_time":\{"type":"integer"\}|"create_time":\{"type":"integer"\}.*"update_time":\{"type":"integer"\})/



=== TEST 16: get proto schema to check if it contains `create_time` and `update_time`
--- request
GET /apisix/admin/schema/proto
--- response_body eval
qr/("update_time":\{"type":"integer"\}.*"create_time":\{"type":"integer"\}|"create_time":\{"type":"integer"\}.*"update_time":\{"type":"integer"\})/



=== TEST 17: get stream_route schema to check if it contains `create_time` and `update_time`
--- request
GET /apisix/admin/schema/stream_route
--- response_body eval
qr/("update_time":\{"type":"integer"\}.*"create_time":\{"type":"integer"\}|"create_time":\{"type":"integer"\}.*"update_time":\{"type":"integer"\})/
