#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
use t::APISIX 'no_plan';

repeat_each(1);
no_long_string();
no_root_location();
no_shuffle();

add_block_preprocessor(sub {
    my ($block) = @_;

    $block->set_value("stream_conf_enable", 1);

    if (!defined $block->extra_stream_config) {
        my $stream_config = <<_EOC_;
    server {
        listen unix:\$TEST_NGINX_HTML_DIR/nginx.sock;

        content_by_lua_block {
            local ext = require("lib.ext-plugin")
            ext.go({})
        }
    }

_EOC_
        $block->set_value("extra_stream_config", $stream_config);
    }

    my $unix_socket_path = $ENV{"TEST_NGINX_HTML_DIR"} . "/nginx.sock";
    my $cmd = $block->ext_plugin_cmd // "['sleep', '5s']";
    my $extra_yaml_config = <<_EOC_;
ext-plugin:
    path_for_test: $unix_socket_path
    cmd: $cmd
_EOC_

    $block->set_value("extra_yaml_config", $extra_yaml_config);

    if (!$block->request) {
        $block->set_value("request", "GET /t");
    }

    if (!$block->error_log) {
        $block->set_value("no_error_log", "[error]\n[alert]");
    }
});

run_tests;

__DATA__

=== TEST 1: add route
--- config
    location /t {
        content_by_lua_block {
            local json = require("toolkit.json")
            local t = require("lib.test_admin")

            local code, message, res = t.test('/apisix/admin/routes/1',
                ngx.HTTP_PUT,
                 [[{
                    "uri": "/hello",
                    "plugins": {
                        "ext-plugin-pre-req": {
                        }
                    },
                    "upstream": {
                        "nodes": {
                            "127.0.0.1:1980": 1
                        },
                        "type": "roundrobin"
                    }
                }]]
            )

            if code >= 300 then
                ngx.status = code
                ngx.say(message)
                return
            end

            ngx.say(message)
        }
    }
--- response_body
passed



=== TEST 2: var
--- request
GET /hello?x=
--- extra_stream_config
    server {
        listen unix:$TEST_NGINX_HTML_DIR/nginx.sock;

        content_by_lua_block {
            local ext = require("lib.ext-plugin")
            local actions = {
                {type = "var", name = "server_addr", result = "127.0.0.1"},
                {type = "var", name = "remote_addr", result = "127.0.0.1"},
                {type = "var", name = "route_id", result = "1"},
                {type = "var", name = "arg_x", result = ""},
            }
            ext.go({extra_info = actions, stop = true})
        }
    }
--- error_code: 405
--- grep_error_log eval
qr/send extra info req successfully/
--- grep_error_log_out
send extra info req successfully
send extra info req successfully
send extra info req successfully
send extra info req successfully



=== TEST 3: ask nonexistent var
--- request
GET /hello
--- more_headers
X-Change: foo
X-Delete: foo
--- extra_stream_config
    server {
        listen unix:$TEST_NGINX_HTML_DIR/nginx.sock;

        content_by_lua_block {
            local ext = require("lib.ext-plugin")
            local actions = {
                {type = "var", name = "erver_addr"},
            }
            ext.go({extra_info = actions, rewrite = true})
        }
    }
--- response_body
uri: /uri
host: localhost
x-add: bar
x-change: bar
x-real-ip: 127.0.0.1
--- grep_error_log eval
qr/send extra info req successfully/
--- grep_error_log_out
send extra info req successfully



=== TEST 4: network is down in the middle
--- request
GET /hello
--- extra_stream_config
    server {
        listen unix:$TEST_NGINX_HTML_DIR/nginx.sock;

        content_by_lua_block {
            local ext = require("lib.ext-plugin")
            local actions = {
                {type = "var", name = "server_addr", result = "127.0.0.1"},
                {type = "closed"},
            }
            ext.go({extra_info = actions, stop = true})
        }
    }
--- error_code: 503
--- error_log
failed to receive RPC_HTTP_REQ_CALL: closed



=== TEST 5: ask response body (not exist)
--- request
GET /hello
--- extra_stream_config
    server {
        listen unix:$TEST_NGINX_HTML_DIR/nginx.sock;

        content_by_lua_block {
            local ext = require("lib.ext-plugin")
            local actions = {
                {type = "respbody", result = nil}
            }
            ext.go({extra_info = actions})
        }
    }
--- error_log: failed to read response body: not exits



=== TEST 6: add route with ext-plugin-post-resp
--- config
    location /t {
        content_by_lua_block {
            local json = require("toolkit.json")
            local t = require("lib.test_admin")

            local code, message, res = t.test('/apisix/admin/routes/1',
                ngx.HTTP_PUT,
                 [[{
                    "uri": "/*",
                    "plugins": {
                        "ext-plugin-post-resp": {
                        }
                    },
                    "upstream": {
                        "nodes": {
                            "127.0.0.1:1980": 1
                        },
                        "type": "roundrobin"
                    }
                }]]
            )

            if code >= 300 then
                ngx.status = code
                ngx.say(message)
                return
            end

            ngx.say(message)
        }
    }
--- response_body
passed



=== TEST 7: ask var
--- request
GET /hello?x=
--- extra_stream_config
    server {
        listen unix:$TEST_NGINX_HTML_DIR/nginx.sock;

        content_by_lua_block {
            local ext = require("lib.ext-plugin")
            local actions = {
                {type = "var", name = "server_addr", result = "127.0.0.1"},
                {type = "var", name = "remote_addr", result = "127.0.0.1"},
                {type = "var", name = "route_id", result = "1"},
                {type = "var", name = "arg_x", result = ""},
            }
            ext.go({extra_info = actions})
        }
    }
--- grep_error_log eval
qr/send extra info req successfully/
--- grep_error_log_out
send extra info req successfully
send extra info req successfully
send extra info req successfully
send extra info req successfully
--- response_body
hello world



=== TEST 8: ask response body
--- request
GET /hello
--- extra_stream_config
    server {
        listen unix:$TEST_NGINX_HTML_DIR/nginx.sock;

        content_by_lua_block {
            local ext = require("lib.ext-plugin")
            local actions = {
                {type = "respbody", result = "hello world\n"},
            }
            ext.go({extra_info = actions})
        }
    }
--- grep_error_log eval
qr/send extra info req successfully/
--- grep_error_log_out
send extra info req successfully
--- response_body
hello world



=== TEST 9: ask response body (chunked)
--- request
GET /hello_chunked
--- extra_stream_config
    server {
        listen unix:$TEST_NGINX_HTML_DIR/nginx.sock;

        content_by_lua_block {
            local ext = require("lib.ext-plugin")
            local actions = {
                {type = "respbody", result = "hello world\n"},
            }
            ext.go({extra_info = actions})
        }
    }
--- grep_error_log eval
qr/send extra info req successfully/
--- grep_error_log_out
send extra info req successfully
--- response_body
hello world



=== TEST 10: ask request body (empty)
--- request
GET /hello
--- extra_stream_config
    server {
        listen unix:$TEST_NGINX_HTML_DIR/nginx.sock;

        content_by_lua_block {
            local ext = require("lib.ext-plugin")
            local actions = {
                {type = "reqbody", result = nil}
            }
            ext.go({extra_info = actions})
        }
    }



=== TEST 11: ask request body
--- request
POST /hello
123
--- extra_stream_config
    server {
        listen unix:$TEST_NGINX_HTML_DIR/nginx.sock;

        content_by_lua_block {
            local ext = require("lib.ext-plugin")
            local actions = {
                {type = "reqbody", result = "123"}
            }
            ext.go({extra_info = actions})
        }
    }
